@@ -3,11 +3,11 @@ NULL
 
 setOldClass("VCorpus")
 
-#' Coerce partition_bundle to VCorpus.
+#' Get VCorpus.
 #' 
-#' Retrieve full text for the \code{partition} objects in a
-#' \code{partition_bundle} and generate a \code{VCorpus}-class object from the
-#' \code{tm}-package.
+#' Retrieve full text for the subcorpora or\code{partition} objects in a
+#' \code{subcorpus_bundle} or \code{partition_bundle} and generate a
+#' \code{VCorpus}-class object from the \code{tm}-package.
 #' 
 #' The \code{VCorpus} class of the \code{tm}-package offers an interface to
 #' access the functionality of the \code{tm}-package. Note however that
@@ -28,24 +28,24 @@ setOldClass("VCorpus")
 #' @name as.VCorpus
 #' @aliases as.VCorpus,partition_bundle-method
 #' @examples
-#' use("polmineR")
-#' p <- partition("GERMAPARLMINI", date = "2009-11-10")
-#' pb <- partition_bundle(p, s_attribute = "speaker")
+#' pb <- partition("GERMAPARLMINI", date = "2009-11-10") %>%
+#'   partition_bundle(s_attribute = "speaker")
+#'  
 #' vc <- as.VCorpus(pb) # works only, if tm-package has not yet been loaded
 #' vc <- as(pb, "VCorpus") # will work if tm-package has been loaded, too
-setMethod("as.VCorpus", "partition_bundle", function(x){
-  as(x, "VCorpus")
-})
+#' 
+#' vc <- corpus("REUTERS") %>% split(s_attribute = "id") %>% as("VCorpus")
+setMethod("as.VCorpus", "partition_bundle", function(x) as(x, "VCorpus") )
 
 #' @name as
 #' @rdname as.VCorpus
 setAs(from = "partition_bundle", to = "VCorpus", def = function(from){
   s_attr_lengths <- sapply(
     s_attributes(from@objects[[1]]@corpus),
-    function(s_attr) CQI$attribute_size(from@objects[[1]]@corpus, s_attr, type = "s")
+    function(s_attr) cl_attribute_size(corpus = from@objects[[1]]@corpus, attribute = s_attr, attribute_type = "s", registry = registry())
   )
   
-  if (length(unique(s_attr_lengths)) == length(s_attr_lengths)){
+  if (length(unique(s_attr_lengths)) == 1L){
     s_attr_to_get <- s_attributes(from@objects[[1]]@corpus)
   } else {
     message("Using only the s-attributes that have the same length as the s-attribute in the slot s_attribute_strucs ",
@@ -53,14 +53,14 @@ setAs(from = "partition_bundle", to = "VCorpus", def = function(from){
     s_attr_to_get <- names(s_attr_lengths[which(s_attr_lengths == s_attr_lengths[from@objects[[1]]@s_attribute_strucs])])
   }
   
-  content <- blapply(
+  content <- lapply(
     from@objects,
-    function(P){
-      metadata <- sapply(s_attr_to_get, function(s_attr) s_attributes(P, s_attr)[1])
+    function(p){
+      metadata <- sapply(s_attr_to_get, function(s_attr) s_attributes(p, s_attr)[1])
       class(metadata) <- "TextDocumentMeta"
       doc <- list(
         meta = metadata,
-        content = get_token_stream(P, p_attribute = "word", collapse = " ")
+        content = get_token_stream(p, p_attribute = "word", collapse = " ")
       )
       class(doc) <- c("PlainTextDocument", "TextDocument")
       doc
